import os
from datetime import datetime
import shutil
import numpy as np
from matplotlib import pyplot as plt
from pandas import DataFrame
import matplotlib.lines as mlines
import pandas

#expids = ['a327','a328','a329','a33u','a33v']
expids = ['a27e_n','a27f_n','a27g_n','a27m_n','a274_n','a27e_r','a27f_r','a27g_r','a27m_r','a274_r']

templatepath = '/gpfs/scratch/bsc32/bsc32786/pisces1d/parameters/variables5wchld.csv'

def make_grid(n):
    y=1
    x=1
    while y*x < n:
        if y==x:
            y=y+1
        else:
            x=x+1
    j=0
    i=0
    tuples=[]
    for a in range(n):
        tuples.append((j,i))
        if i == x - 1:
            i=0
            j=j+1
        else:
            i=i+1

    return y,x,tuples

## get breakpoints
bpfiler = '/gpfs/scratch/bsc32/bsc32340/pisces1d/logs/breakpoints/D5_rand_D5_rand_breaks.csv'
bpdatar = pandas.read_csv(bpfiler)
bplstr = list(bpdatar['gen'])
print(bplstr)

bpfilen = '/gpfs/scratch/bsc32/bsc32340/pisces1d/logs/breakpoints/D5_D5_breaks.csv'
bpdatan = pandas.read_csv(bpfilen)
bplstn = list(bpdatan['gen'])
print(bplstn)

runs = expids

basepath=os.getcwd() + '/'
paraspath='/parameters/'
bbp700path='/bbp700/'
modelfile='PISCES_5d_19600101_19691231_ptrc_T.nc'
obsfile='/gpfs/scratch/bsc32/bsc32786/pisces1d/obs/Mprof_6901486_L75_5days_2015-01-01_2015-12-31reg.nc'

template = pandas.read_csv(templatepath)
parameters=list(template['var'])
defaults=list(template['defaults'])
mins=list(template['min'])
maxes=list(template['max'])
numparams = len(parameters)


Y,X,T = make_grid(numparams)
print(Y,X,T)

cols=['red','red','red','red','red','deepskyblue','deepskyblue','deepskyblue','deepskyblue','deepskyblue']

fig_size = plt.rcParams["figure.figsize"]
fig_size[0] = 12.8
fig_size[1] = 9.6

plt.rcParams["figure.figsize"] = fig_size
plt.locator_params(nbins=6)

fig, axes = plt.subplots(Y, X, sharex=True)
fig.text(0.5, 0.04, 'Generation', ha='center', size='large')

costs={}

for r in range(len(runs)):
    costs[runs[r]]=[]
    stats = []
    for a in range(numparams):
        stats.append([])
    parafilelst = os.listdir(basepath + runs[r] + paraspath)
    parafilelst.sort()

    parasDF = DataFrame(index=range(len(parafilelst)), columns=parameters)

    for f in parafilelst:
        paras = np.genfromtxt(basepath + runs[r] + paraspath + f)
        minind = np.where(paras[:, -1] == min(paras[:, -1]))
        costs[runs[r]].append(float(paras[minind, -1][0][0]))
        for a in range(numparams):
            stats[a].append(paras[minind][0][a])

    for a in range(numparams):
        if r<5:
            axes[T[a][0], T[a][1]].plot(range(len(stats[a])), stats[a], color=cols[r], linewidth=1)#, color='grey', ls='--', linewidth=1)
            axes[T[a][0], T[a][1]].plot(bplstn[r], stats[a][bplstn[r]], color=cols[r], marker='o', zorder=10)
        else:
            axes[T[a][0], T[a][1]].plot(range(len(stats[a])), stats[a], color=cols[r], ls='--', linewidth=0.7)
            axes[T[a][0], T[a][1]].plot(bplstr[r-5], stats[a][bplstr[r-5]], color=cols[r], marker='o', zorder=10)
        parasDF[parameters[a]]=stats[a]

    parasDF.to_csv('parameters_'+runs[r]+'.csv')

for a in range(numparams):
    axes[T[a][0],T[a][1]].plot( range(len(costs[runs[0]])),  [defaults[a]]*len(costs[runs[0]]), color='grey', ls='--', linewidth=1)
    axes[T[a][0],T[a][1]].plot( range(len(costs[runs[0]])),  [mins[a]]*len(costs[runs[0]]), color='grey', ls=':', linewidth=1)
    axes[T[a][0],T[a][1]].plot( range(len(costs[runs[0]])),  [maxes[a]]*len(costs[runs[0]]), color='grey', ls=':', linewidth=1)
    axes[T[a][0], T[a][1]].set_title(parameters[a])

legs=[]


legs.append(mlines.Line2D([], [], color=cols[0], ls='-', label='GA'))
legs.append(mlines.Line2D([], [], color=cols[5], ls='--', label='RS'))
legs.append(mlines.Line2D([], [], color='gray', ls='--', label='Default'))
legs.append(mlines.Line2D([], [], color='gray', ls=':', label='Bounds'))
legs.append(mlines.Line2D([], [], color='white', ls=':',marker='o',markerfacecolor='gray', label='Breakpoint'))

leg1=fig.legend(handles=legs, loc='center right', bbox_to_anchor=(1, 0.5))
plt.gca().add_artist(leg1)

for j in range(len(axes[:,0])):
    for i in range(len(axes[0,:])):
        if not axes[j,i].lines:
            fig.delaxes(axes[j][i])

plt.savefig('parameters.png')
plt.cla()

fig_size = plt.rcParams["figure.figsize"]
fig_size[0] = 6.4
fig_size[1] = 4.8
plt.rcParams["figure.figsize"] = fig_size

fig = plt.figure()


costsDF=DataFrame.from_dict(costs,orient='index').transpose()
costsDF.to_csv('costs.csv')

for r in range(len(runs)):
    if '_n' in runs[r]:
        plt.plot(range(len(costs[runs[r]])), costs[runs[r]], color=cols[0], ls='-', linewidth=1)
        plt.plot(bplstn[r], costs[runs[r]][bplstn[r]], color=cols[r], marker='o',
                 zorder=10)

    else:
        plt.plot( range(len(costs[runs[r]])), costs[runs[r]], color=cols[5], ls='--', linewidth=0.7)
        plt.plot(bplstr[r-5], costs[runs[r]][bplstr[r-5]], color=cols[r], marker='o',
                 zorder=10)

legs=[]

#legs.append(mlines.Line2D([], [], color='gray', ls='--', label='Default'))
# for r in range(len(runs)):
#     legs.append(mlines.Line2D([], [], color=cols[r], ls='-', label=runs[r]))
legs.append(mlines.Line2D([], [], color=cols[0], ls='-', label='GA'))
legs.append(mlines.Line2D([], [], color=cols[5], ls='--', label='RS'))
legs.append(mlines.Line2D([], [], color='white', ls=':',marker='o',markerfacecolor='gray', label='Breakpoint'))
leg1=fig.legend(handles=legs, loc='center right', bbox_to_anchor=(1, 0.5))
plt.gca().add_artist(leg1)
plt.xlabel('Generation')
plt.ylabel('Cost (ST Score)')
plt.savefig('costs.png')

statnames = ['sPOC', 'bPOC']
for statname in statnames:
    fig, axes = plt.subplots(2, 2, sharex=True)
    fig.text(0.5, 0.04, 'Generation', ha='center', size='medium')
    costs={}

    for r in range(len(runs)):

        filelst=sorted([i for i in os.listdir(basepath + runs[r] + '/stats/') if statname in i])

        allbias=[]
        allstd=[]
        allr=[]
        allrmse=[]
        for f in range(len(filelst)):
            data = pandas.read_csv(basepath + runs[r] + '/stats/' + filelst[f])
            #print(data)
            allbias.append(data['bias'][0])
            allstd.append(data['normstd'][0])
            allr.append(data['R'][0])
            allrmse.append(data['RMSE'][0])

        if r<5:
            axes[0,0].plot(allbias, color=cols[r])
            axes[0, 1].plot(allstd, color=cols[r])
            axes[1, 0].plot(allr, color=cols[r])
            axes[1, 1].plot(allrmse, color=cols[r])
            axes[0, 0].plot(bplstn[r], allbias[bplstn[r]], color=cols[r], marker='o', zorder=10)
            axes[0, 1].plot(bplstn[r], allstd[bplstn[r]], color=cols[r], marker='o', zorder=10)
            axes[1, 0].plot(bplstn[r], allr[bplstn[r]], color=cols[r], marker='o', zorder=10)
            axes[1, 1].plot(bplstn[r], allrmse[bplstn[r]], color=cols[r], marker='o', zorder=10)
        else:
            axes[0,0].plot(allbias, color=cols[r], ls='--', linewidth=0.7)
            axes[0, 1].plot(allstd, color=cols[r], ls='--', linewidth=0.7)
            axes[1, 0].plot(allr, color=cols[r], ls='--', linewidth=0.7)
            axes[1, 1].plot(allrmse, color=cols[r], ls='--', linewidth=0.7)
            axes[0, 0].plot(bplstr[r-5], allbias[bplstr[r-5]], color=cols[r], marker='o', zorder=10)
            axes[0, 1].plot(bplstr[r-5], allstd[bplstr[r-5]], color=cols[r], marker='o', zorder=10)
            axes[1, 0].plot(bplstr[r-5], allr[bplstr[r-5]], color=cols[r], marker='o', zorder=10)
            axes[1, 1].plot(bplstr[r-5], allrmse[bplstr[r-5]], color=cols[r], marker='o', zorder=10)

        statsDF = DataFrame(index=range(len(filelst)), columns=['bias','normstd','R','RMSE'])
        statsDF['bias']=allbias
        statsDF['normstd']=allstd
        statsDF['R']=allr
        statsDF['RMSE']=allrmse
        statsDF.to_csv(statname + '_stats_' + runs[r] + '.csv')

    legs=[]

    # legs.append(mlines.Line2D([], [], color='gray', ls='--', label='Default'))
    # for r in range(len(runs)):
    #     legs.append(mlines.Line2D([], [], color=cols[r], ls='-', label=runs[r]))
    legs.append(mlines.Line2D([], [], color=cols[0], ls='-', label='GA'))
    legs.append(mlines.Line2D([], [], color=cols[5], ls='--', label='RS'))
    leg1=fig.legend(handles=legs, loc='center right', bbox_to_anchor=(1, 0.5))
    plt.gca().add_artist(leg1)

    axes[ 0, 0].set_title('Bias',size='medium')
    axes[ 0, 1].set_title('Standard Deviation',size='medium')
    axes[ 1, 0].set_title('Correlation',size='medium')
    axes[ 1, 1].set_title('RMSE',size='medium')

    plt.savefig(statname+'_stats.png')

def avgdeviation(lst, default):
    devs = [abs(i-default) for i in lst]
    a = sum(devs)/len(lst)
    return a